import { getDownloadURL, ref, uploadBytesResumable } from "firebase/storage";
import React, { useState, useEffect, ChangeEvent } from "react";
import { useDispatch, useSelector } from "react-redux";
import { ACTIONS } from "../common/actions";
import { updateCandidate } from "../common/actions/user";
import { fileStorage } from "../common/config";
import { getProfile } from "../common/lib/util";
import { APP_STATE, CandidateType } from "../common/types";
import Avatar from "./Avatar";

const CandidateProfile = () => {
  const dispath = useDispatch();
  const candidate = useSelector(getProfile) as CandidateType;
  const [profile, setProfile] = useState<CandidateType>(candidate);
  const [resume, setResume] = useState<File | null>(null);

  useEffect(() => {
    if (candidate) {
      setProfile(candidate);
    }
  }, [candidate]);

  const onChange = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.name === "phone") {
      setProfile({
        ...profile,
        contacts: { ...profile.contacts, phone: e.target.value },
      });
    } else {
      setProfile({
        ...profile,
        [e.target.name]: e.target.value,
      });
    }
  };

  const onChangeAddress = (e: ChangeEvent<HTMLInputElement>) => {
    setProfile({
      ...profile,
      contacts: {
        ...profile.contacts,
        address: {
          ...profile.contacts.address,
          [e.target.name]: e.target.value,
        },
      },
    });
  };

  const onUpdateCandidate = (payload: Partial<CandidateType>) => {
    dispath(updateCandidate(payload));
  };

  const onSubmit = () => {
    const { jobIds, state, _id, ...rest } = profile;
    dispath({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    if (resume) {
      const resumeRef = ref(fileStorage, `/resumes/${profile._id}.pdf`);
      const uploadTask = uploadBytesResumable(resumeRef, resume);
      uploadTask.on(
        "state_changed",
        (snapshot) => {
          const percent = Math.round(
            (snapshot.bytesTransferred / snapshot.totalBytes) * 100
          );
          console.log(percent);
        },
        (err) => {
          console.log("ERROR ", err);
        },
        () => {
          getDownloadURL(uploadTask.snapshot.ref).then((url) => {
            console.log(url);
            onUpdateCandidate({ ...rest, resume: url });
          });
        }
      );
    } else {
      onUpdateCandidate(rest);
    }
  };

  const onSelectResume = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setResume(e.target.files[0]);
    }
  };

  return (
    <div className="card p-4 mb-3">
      <h5>Edit/Update profile</h5>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Name</label>
        <div className="col-sm-9">
          <input
            type="text"
            className="form-control"
            name="name"
            value={profile.name}
            onChange={onChange}
          />
        </div>
      </div>

      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Address</label>
        <div className="col-sm-9">
          <input
            type="text"
            name="addressLine"
            className="form-control mb-2"
            placeholder="Address line"
            value={profile.contacts?.address?.addressLine}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="City"
            name="city"
            value={profile.contacts?.address?.city}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="Country"
            name="country"
            value={profile.contacts?.address?.country}
            onChange={onChangeAddress}
          />
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Date of birth</label>
            <div className="col-sm-8">
              <input
                type="date"
                className="form-control"
                name="dateOfBirth"
                value={profile?.dateOfBirth}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-3 col-form-label">Phone</label>
            <div className="col-sm-9">
              <input
                type="text"
                className="form-control"
                name="phone"
                value={profile.contacts?.phone}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Resume</label>
            <div className="col-sm-8">
              {profile?.resume && (
                <a
                  className="btn btn-light mb-2"
                  href={profile.resume}
                  target="_blank"
                  rel="noreferrer"
                >
                  Preview
                </a>
              )}
              <input
                type="file"
                className="form-control"
                onChange={onSelectResume}
                accept="application/pdf"
              />
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-3 col-form-label">Self Intro</label>
            <div className="col-sm-9">
              <input type="file" className="form-control" name="oldPassword" />
            </div>
          </div>
        </div>
      </div>
      <div className="mb-3 row">
        <div className="col-sm-6">
          <div className="row">
            <div className="col-sm-4 ">
              <label className="col-form-label">Profile picuture</label>
            </div>
            <div className="col-sm-8">
              <Avatar
                name={profile?.name}
                url={profile?.profilePicture}
                size="lg"
              />
              <button
                type="button"
                className="btn btn-link"
                style={{ padding: "0" }}
              >
                Upload picture
              </button>
            </div>
          </div>
        </div>
      </div>

      <div className="d-md-block">
        <button
          type="button"
          className="btn btn-primary"
          style={{ width: "300px", float: "right" }}
          onClick={onSubmit}
        >
          Update Profile
        </button>
      </div>
    </div>
  );
};

export default CandidateProfile;
