import React, { useEffect, useState, ChangeEvent } from "react";
import { useSelector } from "react-redux";
import { useSearchParams } from "react-router-dom";
import { getStatusColor } from "../common/lib/util";
import {
  JobPayloadType,
  Reducers,
  ApplicationPayloadType,
} from "../common/types";
import Layout from "../components/Layout";

const Applications = () => {
  const jobs = useSelector(
    (state: Reducers) => state.common.jobs
  ) as JobPayloadType[];
  const [searchParams] = useSearchParams();
  const [job, setJob] = useState<JobPayloadType | null>();
  const jobId = searchParams.get("jobId");

  useEffect(() => {
    if (jobId) {
      const found = jobs.find((_job) => _job._id === jobId);
      sortApplications("skills", found);
    }
  }, [jobId, jobs]);

  const onSelectSort = (e: ChangeEvent<HTMLSelectElement>) => {
    sortApplications(e.target.value, job);
  };
  const sortApplications = (sortBy: string, _job?: JobPayloadType | null) => {
    if (_job?.applications) {
      const _applications = _job.applications as ApplicationPayloadType[];
      if (sortBy === "skills") {
        _applications.sort((current, prev) => {
          const skill = current.score.primary + current.score.secondary;
          const prevSkill = prev.score.primary + prev.score.secondary;
          if (skill < prevSkill) {
            return 1;
          } else if (skill > prevSkill) {
            return -1;
          }
          return 0;
        });
      } else if (sortBy === "similarity") {
        _applications.sort((current, prev) => {
          if (current.score.similarity < prev.score.similarity) {
            return 1;
          } else if (current.score.similarity > prev.score.similarity) {
            return -1;
          }
          return 0;
        });
      } else {
        _applications.sort((current, prev) => {
          if (current.score.total < prev.score.total) {
            return 1;
          } else if (current.score.total > prev.score.total) {
            return -1;
          }
          return 0;
        });
      }

      setJob({ ..._job, applications: _applications });
    }
  };

  const renderSkills = (skill: string, index: number) => (
    <span className="skill" key={index}>
      {skill}
    </span>
  );

  const renderApplications = (_application: any, index: number) => {
    const application = _application as ApplicationPayloadType;
    const skillsScore = `PS ${application.score.primary} + SS ${application.score?.secondary} `;
    const status = `badge ${getStatusColor(application.status)}`;

    return (
      <tr key={index}>
        <th scope="row">{application.candidate.name}</th>
        <td>{skillsScore}</td>
        <td>{application.score.similarity}</td>
        <td>{application.score.total}</td>
        <td>
          <span className={status}>{application.status}</span>
        </td>
        <td>{application.candidate.contacts?.phone || ""}</td>
        <td>
          <a
            className="nav-link"
            href={application.candidate.resume}
            target="_blank"
            rel="noreferrer"
          >
            View
          </a>
        </td>
      </tr>
    );
  };

  if (!job) return <Layout title="Applications"></Layout>;
  return (
    <Layout title={job.title}>
      <div className="card p-4 job-preview">
        <p>
          Salary : {job.salary.min} - {job.salary.max} {job.salary.currency}{" "}
        </p>
        <p className="desc">{job.description}</p>
        <label>Primary skills</label>
        <div className="skills">{job.primarySkills.map(renderSkills)}</div>
        <label>Secondary skills</label>
        <div className="skills">{job.secondarySkills.map(renderSkills)}</div>
        <div className="mt-2">
          <label>Applicattions : </label>{" "}
          <a href={`/applications?jobId=${job._id}`}>
            ({job.applications?.length}) Candidates
          </a>
        </div>
      </div>
      <div className="container">
        <div className="row mt-5">
          <h4 className="mb-3">Applications</h4>
          <div className="mb-3 row">
            <label className="col-sm-2 col-form-label">Sort by</label>
            <div className="col-sm-2">
              <select
                className="form-select"
                aria-label="Default select example"
                onChange={onSelectSort}
              >
                <option value="skills">Skills score</option>
                <option value="similarity">Similarity</option>
                <option value="total">Total score</option>
              </select>
            </div>
          </div>

          <div className="card p-4 table-responsive">
            <table className="table table-hover table-sm">
              <thead>
                <tr>
                  <th scope="col">Name</th>
                  <th scope="col">Skills Score</th>
                  <th scope="col">Similarity</th>
                  <th scope="col">Total score</th>
                  <th scope="col">Status</th>
                  <th scope="col">Phone</th>
                  <th scope="col">Resume</th>
                  <th scope="col">Action</th>
                </tr>
              </thead>
              <tbody>{job.applications?.map(renderApplications)}</tbody>
            </table>
          </div>
        </div>
      </div>
    </Layout>
  );
};

export default Applications;
